package BenjaminiHochberg;

use strict;
use warnings;

# Ian Grieve
# September 2006
use Carp;
use Exporter 'import';
use List::Util 'shuffle';

#############################################################
# BenjaminiHochberg module of PoGO.
# Performs the Benjamini-Hochberg Multiple Testing Correction (Benjamini & Hochberg, 1995) on the Fisher's Test p-values to establish significance at selected FDR level.
#############################################################

sub Sort_pvalues
{
    my ($bh_sig_threshold, $clusters_fisher_out_ref) = @_;
    my %clusters_fisher_out = %$clusters_fisher_out_ref;

    my @clusters_details_to_sort = keys(%clusters_fisher_out);
    my @clusters_pvalues;

    foreach my $cluster_details (@clusters_details_to_sort)
    {
	my (@details_split) = split(/\*/, $cluster_details);
	my $current_cluster_name = $details_split[0];
	my $current_cluster_number = $details_split[1];
	my $current_cluster_pvalue = $clusters_fisher_out{$cluster_details};
	
	my $current_cluster_pvaluenumber = $current_cluster_pvalue."*".$current_cluster_number;
	push (@clusters_pvalues, $current_cluster_pvaluenumber);
    }

    my @pvalues_sorted = sort { (split /\*/, $a)[0] <=> (split /\*/, $b) [0] } @clusters_pvalues;
    my @bh_significant_clusters = Benjamini_hochberg($bh_sig_threshold, \@pvalues_sorted);
}

sub Benjamini_hochberg
{
    my ($bh_sig_threshold, $sorted_pvalues_ref) = @_;
    my @pvalues_sorted = @$sorted_pvalues_ref;
    my $bh_m = scalar(@pvalues_sorted);
    my $bh_q = $bh_sig_threshold;
    my $bh_k = 0;
    my $i = 0;
    my @bh_rejected_null = "";
    
    foreach my $bh_array_element (@pvalues_sorted)
    {
	my @bh_array_element_split = split /\*/, $bh_array_element;
	my $bh_pvalue = $bh_array_element_split[0];
	my $bh_clusterno = $bh_array_element_split[1];
	
	my $bh_testvalue = $bh_q * $bh_k / $bh_m;

	if ($bh_testvalue > $bh_pvalue)
	{
	    push (@bh_rejected_null, $bh_k);
	}
	
	$bh_k = $bh_k + 1;
	
    }
    
    my $bh_rejected_max = pop (@bh_rejected_null);
    my @bh_significant_clusters; 

    until ($i == $bh_rejected_max + 1)
    {
	$pvalues_sorted[$i] =~ /(\d+.?\d*)\*(\d+)/;
	$i = $i + 1;
	push (@bh_significant_clusters, $2);
    }
    
    return @bh_significant_clusters;
}

1;
