package PermutationTests;

use strict;
use warnings;

# Ian Grieve
# September 2006
use Carp;
use Exporter 'import';
use List::Util 'shuffle';

#############################################################
# PermutationTests module of PoGO.
# Carries out a permutation test for each POSOC Cluster.
#############################################################

sub Randomize
{    
    my ($random_geneslist_length, @full_genes_list) = @_;
    my @random_genes_list;

    my @genes_list_shuffled = shuffle (@full_genes_list);
    
    for (my $i=1; $i<=$random_geneslist_length; $i++)
    {
	chomp $genes_list_shuffled[$i];
	push (@random_genes_list, $genes_list_shuffled[$i]);
    }

    return (@random_genes_list);
}


sub Cluster_affymatch
{
    my ($permutation_output_filename, $current_iteration, $posoc_cluster_genes, $randomized_list) = @_;
    open (PERMUTATION_DEBUG, ">>$permutation_output_filename") || die("Cannot reopen $permutation_output_filename (permutation debug file) in PoGO, PermutationTests.pm module, Cluster_affymatch subroutine.\n"); 
    my $cluster_count = 0;
    
    print PERMUTATION_DEBUG "\n\n## Iteration $current_iteration ##\n\n";
    foreach my $cluster_geneslist (@$posoc_cluster_genes)
    {
	my $match_count = 0;
	$cluster_count = $cluster_count + 1;
	print PERMUTATION_DEBUG "CLUSTER $cluster_count";
       
	foreach my $randomized_gene (@$randomized_list)
	{
	    chomp $randomized_gene;
	    $randomized_gene =~ /[^=]*/;
	    
	    if ($cluster_geneslist =~ /$randomized_gene/)
	    {
#		print PERMUTATION_DEBUG "Match!\t";
#		print PERMUTATION_DEBUG $randomized_gene;
#		print PERMUTATION_DEBUG "\n";
		$match_count = $match_count + 1;
	    }
	}

	print PERMUTATION_DEBUG ": $match_count\n";
    }

    close (PERMUTATION_DEBUG);
}


sub Vector_maker
{
    my ($vector_iteration, $perminput_split_byiteration) = @_;
    my $vector_counts = 'vector_counts.txt';
    open (VECTOR_COUNTS, ">>$vector_counts") || die ("Cannot reopen $vector_counts vectors output file in PoGO, PermutationTests.pm module, Vector_maker subroutine.\n"); 
    
    print VECTOR_COUNTS "Cluster ";
    print VECTOR_COUNTS $vector_iteration;
    print VECTOR_COUNTS ": ";

    foreach my $perm_iteration (@$perminput_split_byiteration)
    {
	if ($perm_iteration =~ /CLUSTER $vector_iteration:\s+(\d+)/g)
	{
	    print VECTOR_COUNTS $1;
	    print VECTOR_COUNTS ", ";
	}

	else { next; }
    }

    print VECTOR_COUNTS "\n";
    close (VECTOR_COUNTS);
}


sub Vector_comparer
{
    my ($cluster_min_de_genes, $vector_counts_input, $de_counts_in) = @_;
    my $permutation_tests_output = 'permutation_tests.txt';
    open (PERMUTATION_TESTS, ">>$permutation_tests_output") || die ("Cannot open $permutation_tests_output output file in PoGO, PermutationTests.pm module, Vector_comparer subroutine.\n");
    
    my $less_than_counter = 0;
    my $greater_than_counter = 0;
    my ($de_input_cluster_genes, $de_input_cluster_number);
    my %permtest_results = ();

    foreach my $de_input_line (@$de_counts_in)
    {
	if ($de_input_line =~ /CLUSTER (\d+): (\d+)/)
	{
	    $de_input_cluster_number = $1;
	    $de_input_cluster_genes = $2;
	    
	    if ($de_input_cluster_genes >= $cluster_min_de_genes)
	    {
		print PERMUTATION_TESTS "\n\n##\n$de_input_cluster_number\t$de_input_cluster_genes\n\n";
	    }
	#    next;
	
	
	foreach my $vector_counts_line (@$vector_counts_input)
	{
	    if ($vector_counts_line =~ /Cluster $de_input_cluster_number:(.*)/)
	    {
		my $vector_input_cluster_genes = $1;
		my @vector_input_cluster_genes_tosplit = split(/, /, $vector_input_cluster_genes);
		
		foreach my $vector_genes_count (@vector_input_cluster_genes_tosplit)
		{
		    if ($vector_genes_count =~ /(\d+)/) # Don't test anything that's not a number (i.e. the blank space after the last comma in the vector).
		    {
			if ($1 < $de_input_cluster_genes) # The count of random-set matches to the cluster is smaller than the DE count
			{
			    $less_than_counter = $less_than_counter + 1; 
			}
			
			elsif ($1 > $de_input_cluster_genes) # The count of random-set matches to the cluster is greater than the DE count
			{
			    $greater_than_counter = $greater_than_counter + 1;
			}
			
			elsif ($1 == $de_input_cluster_genes)
			{
			    $less_than_counter = $less_than_counter + 1;
			    $greater_than_counter = $greater_than_counter + 1;
			}
			
			else
			{
			    print PERMUTATION_TESTS "Error!";
			    die ("A mathemetical error has occurred during vector comparison.\n");
			}
		    }
		}
	    
		my $cluster_counts_total = $less_than_counter + $greater_than_counter;
		my ($de_overrep_pvalue, $de_underrep_pvalue);
		
		if ($cluster_counts_total > 0)
		{
		    $de_overrep_pvalue = $less_than_counter / $cluster_counts_total; # I realise this is counterintuitive. If there are fewer matches to the cluster among the null sets of Affy IDs (in the vector) than among the real DE Affy IDs, then DE Affy IDs can be considered to be 'overrepresented' in the cluster.
		}
		else 
		{
		    $de_overrep_pvalue = 1;
		}
		
		if ($cluster_counts_total > 0)
		{
		    $de_underrep_pvalue = $greater_than_counter / $cluster_counts_total;
		}
		else 
		{
		    $de_underrep_pvalue = 1;
		}
		
		if ($de_input_cluster_genes >= $cluster_min_de_genes)
		{
		    my @pvalues_array = ($de_input_cluster_genes, $de_overrep_pvalue, $de_underrep_pvalue);
		    $permtest_results{$de_input_cluster_number} = \@pvalues_array;
		    
		    print PERMUTATION_TESTS "P-value for the probability that DE genes are under-represented in Cluster ";
		    print PERMUTATION_TESTS $de_input_cluster_number;
		    print PERMUTATION_TESTS ": ";
		    printf PERMUTATION_TESTS ("%.5f", $de_underrep_pvalue);
		    print PERMUTATION_TESTS "\n";
		    print PERMUTATION_TESTS "P-value for the probability that DE genes are over-represented in Cluster ";
		    print PERMUTATION_TESTS $de_input_cluster_number;
		    print PERMUTATION_TESTS ": ";
		    printf PERMUTATION_TESTS ("%.5f", $de_overrep_pvalue);
		    print PERMUTATION_TESTS "\n";
		}
		
		if ($de_overrep_pvalue < 0.05 && $de_input_cluster_genes > $cluster_min_de_genes)
		{
		    print PERMUTATION_TESTS "The number of DE genes in Cluster ";
		    print PERMUTATION_TESTS $de_input_cluster_number;
		    print PERMUTATION_TESTS " is significantly fewer than expected";
		    print PERMUTATION_TESTS "\n\n";
		}
		
		elsif ($de_underrep_pvalue < 0.05 && $de_input_cluster_genes > $cluster_min_de_genes)
		{
		    print PERMUTATION_TESTS "The number of DE genes in Cluster ";
		    print PERMUTATION_TESTS $de_input_cluster_number;
		    print PERMUTATION_TESTS " is significantly greater than expected";
		    print PERMUTATION_TESTS "\n\n";
		}
		
		elsif ($de_input_cluster_genes > $cluster_min_de_genes) 
		{
		    print PERMUTATION_TESTS "\n\n";
		}
		
		$less_than_counter = 0;
		$greater_than_counter = 0;
		
	    }
	}
	}
    }
    
    close (PERMUTATION_TESTS);
    return %permtest_results;
}

1;
